require 'thor'

require 'ptf/client/task'
require 'ptf/client/group'
require 'ptf/client/summary'

module Ptf
  module Cli
    class Application < Thor

      desc "init", "Initialize a new Planning Text File project"
      long_desc <<-LONGDESC
        Initializes a new Planning Text File workspace.

        With the -f option it writes over an existing PTF workspace.
      LONGDESC
      option :f, :type => :boolean
      def init
        puts Ptf::Commands::Init.init(options[:f])
      end

      desc "task", "Subcommand to manage tasks"
      subcommand "task", Task

      desc "group", "Subcommand to manage groups"
      subcommand "group", Group

      desc "list DELIMIT", "List tasks. Given 'all' or a group name"
      option :open, :type => :boolean, :aliases => :o
      option :closed, :type => :boolean, :aliases => :c
      def list(delimiter)
        puts Ptf::Commands::List.list(delimiter, options[:open], options[:closed])
      end

      desc "summary", "Subcommand to manage a summary of all tasks"
      subcommand "summary", Summary

    end
  end
end

