module Ptf
  class DataFile

    DIR_PATH = Ptf::FileSystem.data_dir

    # Initialize a new DataFile.
    #
    # @param metadata_file [Ptf::MetadataFile] the associated metadata file.
    #
    # @raise [ArgumntError] if a MetadataFile is not given.
    def initialize(metadata_file)
      raise ArgumentError, "Metadata file not given." unless metadata_file.is_a? Ptf::MetadataFile

      @metadata_file = metadata_file
    end

    # Returns the full path to the data file.
    #
    # @return [String] the full path to the data file.
    def data_file_path
      File.join(DIR_PATH, @metadata_file.hash)
    end

    # Returns the header string for the file.
    #
    # @return [String] the header string for the data file.
    def header_string
      "##{@metadata_file.group.abbreviation}--#{@metadata_file.id} #{@metadata_file.title}\n#Due: #{@metadata_file.due_date_str}\n#Estimate: #{@metadata_file.estimate}\n\n\n"
    end

    # Write the header string to the data file. Overwrites any data file for the same task.
    def write_to_file
      File.open(data_file_path, "w") do |f|
        f.write header_string
      end
    end

  end
end

