module Ptf
  module FileSystem
    class << self

      def file_exist?(filepath)
        File.file?(filepath)
      end

      def config
        @config = Ptf::Config.get_config if @config.nil?

        @config
      end

      def file_permission
        @config[:file_permission]
      end

      def base_dir
        config[:base_dir]
      end

      def id_counter_file
        File.join(base_dir, config[:task_counter_file])
      end

      def id_counter_starting_content
        config[:id_counter_start]
      end

      def group_list_file
        File.join(base_dir, config[:group_list_file])
      end

      def metadata_dir
        File.join(base_dir, config[:task_metadata_dir])
      end

      def metadata_open_dir
        File.join(metadata_dir, config[:in_progress_dir])
      end

      def metadata_closed_dir
        File.join(metadata_dir, config[:completed_dir])
      end

      def data_dir
        File.join(base_dir, config[:task_data_dir])
      end

      def tmp_dir
        File.join(base_dir, config[:tmp_dir])
      end

      def id_exist?(id)
        next_id = File.read(id_counter_file).to_i

        (id < next_id && id > 0)
      end

      def find_file(id, open = true)
        raise ArgumentError, "No task with id #{id} exists." unless id_exist?(id)

        search_dir = (open ? metadata_open_dir : metadata_closed_dir)
        groups = Ptf::Group.all_groups

        groups.each do |g|
          dir = File.join(search_dir, g.name)
          possible_id_file = File.join(dir, id.to_s)

          if file_exist?(possible_id_file)
            return possible_id_file
          end
        end

        nil
      end

    end
  end
end
