module Ptf
  class Group

    def initialize(name, abbrev)
      @name = name
      @abbreviation = abbrev
    end

    def name
      @name
    end

    def abbreviation
      @abbreviation
    end

    def to_s
      "#{name}:#{abbreviation}"
    end

    def is_default?
      Group.default_group.to_s == to_s
    end


    class << self
      def config
        @config = Ptf::Config.get_config if @config.nil?

        @config
      end

      def from_name(group_name)
        group_file = Ptf::FileSystem.group_list_file
        File.open(group_file, "r") do |f|
          f.each_line do |l|
            name, abbrev = l.gsub(/\s+/, "").split(":")

            if group_name == name
              return Group.new(name, abbrev)
            end

          end
        end

        nil
      end

      def from_abbrev(group_abbrev)
        group_file = Ptf::FileSystem.group_list_file
        File.open(group_file, "r") do |f|
          f.each_line do |l|
            name, abbrev = l.gsub(/\s+/, "").split(":")

            if group_abbrev == abbrev
              return Group.new(name, abbrev)
            end

          end
        end

        nil
      end

      def get_group(group)
        if(group.length <= 4 && group == group.upcase)
          from_abbrev(group)
        else
          from_name(group)
        end
      end

      def group_exist?(g)
        group_file = Ptf::FileSystem.group_list_file

        File.open(group_file, "r") do |f|
          f.each_line do |l|
            name, abbrev = l.gsub(/\s+/, "").split(":")

            if name == g || abbrev == g
              return true
            end

          end
        end

        false
      end

      def all_groups
        group_arr = []

        File.open(Ptf::FileSystem.group_list_file, "r") do |f|
          f.each_line do |l|
            name, abbrev = l.gsub(/\s+/, "").split(":")

            group_arr.push (Ptf::Group.new(name, abbrev))
          end
        end

        group_arr
      end

      def default_group
        default_group = config[:default_group]

        name, abbrev = default_group.gsub(/\s+/, "").split(":")
        Ptf::Group.new(name, abbrev)
      end

    end

  end
end

